/*
 * Simple ICMP packet logger by Zucco (zucco@netposta.net)
 * http://w3.swi.hu/zucco/
 * BUGS: drop_promisc() - ioctl hiba. 
 * thanks to: Chad Renfro, Xphere.
 * -------------------------------
 * s1lent is bad! leave!
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <netdb.h>
#include <net/if.h>
#include <sys/types.h>
#include <linux/if_ether.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <netinet/ether.h>
#include <netinet/ip_icmp.h>
#include <syslog.h>
#include <stdarg.h>

void term(int sig);
void daemon_init();
void log_it(char *szoveg, ...);

int main (int argc, char **argv)  {
 int sock, bytes_read, fromlen;
 char buffer[65535];
 struct sockaddr_in from;
 struct ip *ip;
 struct icmp *icmp;
 const char *dev = "eth0";
 
 if (getuid() || getgid()) {
   fprintf(stderr,"Run only with root\n");
   exit(0);
 }
 
 sock = open_psocket();
 if (argv[1]) {
   setpromisc(sock, argv[1]);
 } else {
   setpromisc(sock, dev);
 }
 
 daemon_init();
 log_it("Starting (pid=%d).", getpid());

 while (sock) {
   fromlen = sizeof from;
   bytes_read = recvfrom(sock, buffer, sizeof buffer, 0, (struct sockaddr *)&from, 
   &fromlen);
   ip = (struct ip *)(buffer + sizeof(struct ethhdr));
   icmp = (struct icmp *)(buffer + sizeof(struct ethhdr) + (ip->ip_hl<<2));
   if (icmp->icmp_type > 0 && icmp->icmp_type < 19) {
     log_it("packet (%d) from %s", icmp->icmp_type, inet_ntoa(from.sin_addr));
   }
   signal(SIGILL, term);
   signal(SIGBUS, term);
   signal(SIGFPE, term);
   signal(SIGSTOP, term);
   signal(SIGINT, term);
   signal(SIGKILL, term);
   signal(SIGQUIT, term);
   signal(SIGTERM, term);
   signal(SIGSEGV, term);
 }
}

int open_psocket () {
  int sock;
  if((sock = socket(PF_INET, SOCK_PACKET, htons(0x0003))) < 0) {
    perror("socket");
    exit(0);
  }
}

int setpromisc (int fd, const char *dev) {
  struct ifreq ifr;
  strcpy(ifr.ifr_name, dev);
  if (ioctl(fd, SIOCGIFFLAGS, &ifr) < 0) {
    perror("ioctl");
    exit(errno);
  }
  ifr.ifr_flags |= IFF_PROMISC;
  if (ioctl(fd, SIOCSIFFLAGS, &ifr) < 0) {
    perror("ioctl");
    exit(errno);
  }
}

int drop_promisc (int fd, const char *dev) {
  struct ifreq ifr;
  if (!(ifr.ifr_flags & IFF_PROMISC))
    printf("Promisc mode not set!\n");
  ifr.ifr_flags &= ~IFF_PROMISC;
  if (ioctl(fd, SIOCSIFFLAGS, &ifr) < 0) {
    perror("ioctl");
    exit(errno);
  }
}

void term (int sig) {
  int sock;
  const char *dev;
  log_it("Received signal: %d, exiting.\n", sig);
  closelog();
  drop_promisc(sock, dev);
  exit(0);
}

void daemon_init() {
  printf("Forking to background...\n");
  if (fork ())
    exit(0);
  setsid();
  
  if (fork ())
    exit(0);
    
  close(0); // error
  close(1); // output
  close(2); // input
  signal(SIGHUP, SIG_IGN);
}

void log_it(char *szoveg, ...) {
  va_list arglist;
  char text[256];
  
  va_start(arglist, szoveg);
  vsnprintf(text, 256, szoveg, arglist);
  va_end(arglist);
   
  syslog(LOG_DAEMON|LOG_INFO, "%s", text); // break format string bug ;)
}
  
